import { DocumentModel, Node, BuiltinSimulatorRenderer } from '@alilc/lowcode-designer';
import Vue, { ComponentPublicInstance, Component } from 'vue';
import { I18nMessages, Config } from '@mfejs/lowcode-vue-renderer2.7';
export * from '@mfejs/lowcode-vue-renderer2.7';
export { default as VueRenderer, config as vueRendererConfig } from '@mfejs/lowcode-vue-renderer2.7';
import Router from 'vue-router';
import { IPublicTypeRootSchema, IPublicTypeNpmInfo, IPublicTypeComponentSchema } from '@alilc/lowcode-types';

interface ComponentRecord {
    did: string;
    nid: string;
    cid: number;
}

type ComponentInstance = ComponentPublicInstance;
interface DocumentInstance {
    readonly id: string;
    readonly key: string;
    readonly path: string;
    readonly scope: Record<string, unknown>;
    readonly document: DocumentModel;
    readonly instancesMap: Map<string, ComponentInstance[]>;
    readonly schema: IPublicTypeRootSchema;
    readonly messages: I18nMessages;
    getComponentInstance(id: number): ComponentInstance | null;
    mountInstance(id: string, instance: ComponentInstance): (() => void) | void;
    unmountIntance(id: string, instance: ComponentInstance): void;
    rerender(): void;
    getNode(id: string): Node | null;
}
type MinxedComponent = IPublicTypeNpmInfo | Component | IPublicTypeComponentSchema;
interface SimulatorViewLayout {
    Component?: Component;
    componentName?: string;
    props?: Record<string, unknown>;
}

interface VueSimulatorRenderer extends BuiltinSimulatorRenderer {
    app: Vue;
    config: Config;
    router: Router;
    layout: SimulatorViewLayout;
    device: string;
    locale: string;
    designMode: 'design';
    libraryMap: Record<string, string>;
    components: Record<string, Component>;
    autoRender: boolean;
    componentsMap: Record<string, MinxedComponent>;
    documentInstances: DocumentInstance[];
    requestHandlersMap: Record<string, CallableFunction>;
    dispose(): void;
    rerender(): void;
    getCurrentDocument(): DocumentInstance | undefined;
}

declare const _default: VueSimulatorRenderer;

export { type ComponentInstance, type ComponentRecord, type DocumentInstance, type MinxedComponent, type SimulatorViewLayout, type VueSimulatorRenderer, _default as default };
