var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import Vue, { defineComponent, h, shallowRef, reactive, markRaw, ref, computed, onUnmounted } from "vue";
import { isElement, isObject, isNil, AssetLoader, getSubComponent, buildUtils, buildComponents } from "@mfejs/lowcode-utils2.7";
import VueRouter, { RouterView } from "vue-router";
import LowCodeRenderer__default, { config } from "@mfejs/lowcode-vue-renderer2.7";
export * from "@mfejs/lowcode-vue-renderer2.7";
import { default as default2, config as config2 } from "@mfejs/lowcode-vue-renderer2.7";
const host = window.LCSimulatorHost;
class Cursor {
  constructor() {
    __publicField(this, "states", /* @__PURE__ */ new Set());
  }
  setDragging(flag) {
    if (flag) {
      this.addState("dragging");
    } else {
      this.removeState("dragging");
    }
  }
  setXResizing(flag) {
    if (flag) {
      this.addState("x-resizing");
    } else {
      this.removeState("x-resizing");
    }
  }
  setYResizing(flag) {
    if (flag) {
      this.addState("y-resizing");
    } else {
      this.removeState("y-resizing");
    }
  }
  setCopy(flag) {
    if (flag) {
      this.addState("copy");
    } else {
      this.removeState("copy");
    }
  }
  isCopy() {
    return this.states.has("copy");
  }
  release() {
    for (const state of this.states)
      this.removeState(state);
  }
  addState(state) {
    if (!this.states.has(state)) {
      this.states.add(state);
      document.documentElement.classList.add(`lc-cursor-${state}`);
    }
  }
  removeState(state) {
    if (this.states.has(state)) {
      this.states.delete(state);
      document.documentElement.classList.remove(`lc-cursor-${state}`);
    }
  }
}
const cursor = new Cursor();
let nativeSelectionEnabled = true;
const preventSelection = (e) => {
  if (nativeSelectionEnabled) {
    return null;
  }
  e.preventDefault();
  e.stopPropagation();
  return false;
};
document.addEventListener("selectstart", preventSelection, true);
document.addEventListener("dragstart", preventSelection, true);
function setNativeSelection(enableFlag) {
  nativeSelectionEnabled = enableFlag;
}
const cycleRange = document.createRange();
function getClientRects(node) {
  if (!node.parentNode)
    return [];
  if (isElement(node)) {
    return [node.getBoundingClientRect()];
  }
  cycleRange.selectNode(node);
  return Array.from(cycleRange.getClientRects());
}
function isCommentNode(el) {
  return el.nodeType === 8;
}
function isTextNode(el) {
  return el.nodeType === 3;
}
function isDomNode(el) {
  return isObject(el) && "nodeType" in el && (el.nodeType === Node.ELEMENT_NODE || el.nodeType === Node.TEXT_NODE);
}
function isEmptyNode(el) {
  return isCommentNode(el) || isTextNode(el) && el.nodeValue === "";
}
const SYMBOL_VDID = Symbol("_LCDocId");
const SYMBOL_VNID = Symbol("_LCNodeId");
const SYMBOL_VInstance = Symbol("_LCVueInstance");
const SYMBOL_RECORD_FLAG = Symbol("_LCVueCompRecord");
function isVNodeHTMLElement(el) {
  return isObject(el) && !isNil(el.__vueParentComponent);
}
function isCompRootHTMLElement(el) {
  return isObject(el) && SYMBOL_VDID in el;
}
function isComponentRecord(el) {
  return isObject(el) && SYMBOL_RECORD_FLAG in el;
}
function getCompRootData(el) {
  return {
    docId: el[SYMBOL_VDID],
    nodeId: el[SYMBOL_VNID],
    instance: el[SYMBOL_VInstance]
  };
}
function setCompRootData(el, data) {
  el[SYMBOL_VDID] = data.docId;
  el[SYMBOL_VNID] = data.nodeId;
  el[SYMBOL_VInstance] = data.instance;
}
function createComponentRecord(did, nid, cid) {
  return {
    did,
    nid,
    cid,
    [SYMBOL_RECORD_FLAG]: true
  };
}
function findDOMNodes(instance) {
  const els = [];
  const el = instance.$el;
  if (isEmptyNode(el)) {
    appendSiblingElement(els, instance, el, (c) => c.previousSibling);
    appendDescendantComponent(els, instance);
    appendSiblingElement(els, instance, el, (c) => c.nextSibling);
  } else {
    els.push(el);
  }
  return els;
}
function appendSiblingElement(target, instance, el, next) {
  let nextNode = next(el);
  for (; nextNode; ) {
    if (isEmptyNode(nextNode)) {
      nextNode = next(nextNode);
      continue;
    }
    if (isVNodeHTMLElement(nextNode)) {
      const childInstance = nextNode.__vueParentComponent;
      if (isChildInstance(instance, childInstance)) {
        target.unshift(nextNode);
        nextNode = next(nextNode);
        continue;
      }
    }
    break;
  }
}
function appendDescendantComponent(target, instance) {
  const subNode = instance.$vnode;
  const current = subNode.elm;
  if (isValidElement(current)) {
    target.push(current);
    return true;
  }
  if (Array.isArray(subNode.children) && subNode.children.length > 0) {
    return appendDescendantChildren(target, subNode.children);
  } else if (subNode.componentInstance) {
    return appendDescendantComponent(target, subNode.componentInstance);
  }
  return false;
}
function appendDescendantChildren(target, children) {
  const validElements = children.map(({ elm }) => elm).filter(isValidElement);
  if (validElements.length > 0) {
    target.push(...validElements);
    return true;
  } else {
    return children.length > 0 && children.some((item) => {
      if (Array.isArray(item.children) && item.children.length > 0) {
        return appendDescendantChildren(target, item.children.filter(isValidElement));
      } else if (item.componentInstance) {
        return appendDescendantComponent(target, item.componentInstance);
      }
      return false;
    });
  }
}
function isValidElement(el) {
  if (el && isDomNode(el) && !isEmptyNode(el)) {
    const rect = getClientRects(el);
    return rect.some((item) => item.width || item.height);
  }
  return false;
}
function isChildInstance(target, source) {
  if (source == null)
    return false;
  if (target._uid > source._uid)
    return false;
  if (target._uid === source._uid)
    return true;
  return isChildInstance(target, source.$parent);
}
function getClosestNodeInstance(el, specId) {
  if (!document.contains(el)) {
    console.warn("当前元素已经不存在", el);
    return null;
  }
  if (isVNodeHTMLElement(el)) {
    const e = el.__vueParentComponent;
    return getClosestNodeInstanceByComponent(e, specId);
  }
  return getClosestNodeInstanceByElement(el, specId);
}
function getClosestNodeInstanceByElement(el, specId) {
  for (; el; ) {
    if (isCompRootHTMLElement(el)) {
      const { nodeId, docId, instance } = getCompRootData(el);
      if (!specId || specId === nodeId) {
        return {
          docId,
          nodeId,
          instance: createComponentRecord(docId, nodeId, instance._uid)
        };
      }
    }
    el = el.parentElement;
  }
  return null;
}
function getClosestNodeInstanceByComponent(instance, specId) {
  for (; instance; ) {
    const el = instance.$el;
    if (el && isCompRootHTMLElement(el)) {
      const { nodeId, docId, instance: instance2 } = getCompRootData(el);
      if (!specId || specId === nodeId) {
        return {
          docId,
          nodeId,
          instance: createComponentRecord(docId, nodeId, instance2._uid)
        };
      }
    }
    instance = instance.$parent;
  }
  return null;
}
const Renderer = defineComponent({
  name: "Renderer",
  props: {
    simulator: {
      type: Object,
      required: true
    },
    documentInstance: {
      type: Object,
      required: true
    }
  },
  render() {
    const { documentInstance, simulator: simulator2 } = this;
    const { schema, scope, messages, key } = documentInstance;
    const { designMode, device, locale, components } = simulator2;
    return simulator2.autoRender ? h(LowCodeRenderer__default, {
      key,
      props: {
        scope,
        schema,
        locale,
        device,
        messages,
        components,
        designMode,
        getNode: (id) => documentInstance.getNode(id),
        /**
         * 组件 ctx 更新的回调，在组件每次 render 渲染周期我们都会为组件构造新的上下文环境，因此该回调函数会在组件每次 render 过程中触发，主要包含两个参数：
         * - `schema`：当前组件的 schema 模型结构
         * - `ctx`：当前组件的上下文信息，主要包含以下内容：
         *    - `page`：当前页面容器实例
         *    - `this`: 当前组件所属的容器组件实例
         *    - `item`/`index`: 循环上下文（属性 key 可以根据 loopArgs 进行定制）
         *    - `form`: 表单上下文
         *
         * @param schema
         * @param ref
         * @returns
         */
        onCompGetCtx: (schema2, ref2) => documentInstance.mountInstance(schema2.id, ref2)
      }
    }) : null;
  }
});
const Layout = defineComponent({
  name: "Layout",
  props: {
    simulator: {
      type: Object,
      required: true
    }
  },
  render() {
    const { simulator: simulator2, $slots } = this;
    const { layout, getComponent } = simulator2;
    if (layout) {
      const { Component, props = {}, componentName } = layout;
      if (Component)
        return h(
          Component,
          { props: { ...props, simulator: simulator2 }, key: "layout" },
          $slots.default
        );
      const component = componentName && getComponent(componentName);
      if (component)
        return h(
          component,
          { props: { ...props, simulator: simulator2 }, key: "layout" },
          $slots.default
        );
    }
    return h("div", $slots.default);
  }
});
const SimulatorRendererView = defineComponent({
  name: "SimulatorRendererView",
  props: {
    simulator: {
      type: Object,
      required: true
    }
  },
  render() {
    const { simulator: simulator2 } = this;
    return h(
      Layout,
      {
        props: {
          simulator: simulator2
        }
      },
      [h(RouterView)]
    );
  }
});
function deepMerge(o1, o2) {
  if (isObject(o1) && isObject(o2)) {
    const result = Object.assign({}, o1);
    Object.keys(o2).forEach((key) => {
      Reflect.set(result, key, deepMerge(o1[key], o2[key]));
    });
    return result;
  }
  return o2 != null ? o2 : o1;
}
const Leaf = defineComponent({
  functional: true,
  render(h2, context) {
    return context.scopedSlots.default({});
  }
});
Object.assign(Leaf, {
  displayName: "Leaf",
  componentMetadata: {
    componentName: "Leaf",
    configure: {
      props: [
        {
          name: "children",
          setter: "StringSetter"
        }
      ],
      supports: false
    }
  }
});
function normalizeComponentAttrs(component) {
  if (typeof component != "object" || typeof component.props == "object" || typeof component.on == "object" || typeof component.attrs == "object") {
    return component;
  }
  const normalizedOn = Object.keys(component).filter((key) => key.match(/^on[A-Z]/)).reduce((onObj, key) => {
    let eventName = key.slice(2);
    eventName && (eventName = eventName[0].toLowerCase() + eventName.slice(1));
    onObj[eventName] = component[key];
    delete component[key];
    return onObj;
  }, {});
  const remainingAttrs = Object.keys(component).reduce((attrsObj, key) => {
    if (!attrs.includes(key)) {
      attrsObj[key] = component[key];
    }
    return attrsObj;
  }, {});
  return {
    ...component,
    on: normalizedOn,
    attrs: remainingAttrs
  };
}
const attrs = [
  "key",
  "slot",
  "ref",
  "is",
  "pre",
  "tag",
  "staticClass",
  "class",
  "staticStyle",
  "style",
  "normalizedStyle",
  "props",
  "attrs",
  "domProps",
  "hook",
  "on",
  "nativeOn",
  "transition",
  "inlineTemplate",
  "directives",
  "scopedSlots"
];
function createVNode(componentInstance, props, slotContent) {
  const normalizedProps = normalizeComponentAttrs(props || {});
  if (componentInstance && componentInstance.__htmlComponent) {
    normalizedProps.attrs = normalizedProps.props;
    delete normalizedProps.props;
  }
  let slotFn;
  if (slotContent && typeof slotContent.default == "function") {
    slotFn = slotContent.default();
  }
  if (slotContent && typeof slotContent == "object") {
    const scopedSlots = {};
    Object.keys(slotContent).forEach((slotKey) => {
      if (slotKey !== "default") {
        const slotValue = slotContent[slotKey];
        if (slotValue && typeof slotValue == "function") {
          slotValue.proxy = true;
          scopedSlots[slotKey] = slotValue;
        }
      }
    });
    if (Object.keys(scopedSlots).length > 0) {
      normalizedProps.scopedSlots = scopedSlots;
    }
  }
  return h(componentInstance, normalizedProps, slotFn);
}
const Slot = defineComponent({
  setup(props, { slots }) {
    return () => createVNode("div", { class: "lc-container" }, slots);
  }
});
Object.assign(Slot, {
  displayName: "Slot",
  componentMetadata: {
    componentName: "Slot",
    configure: {
      props: [
        {
          name: "___title",
          title: "插槽标题",
          setter: "StringSetter",
          defaultValue: "插槽容器"
        },
        {
          name: "___params",
          title: "插槽入参",
          setter: {
            componentName: "ArraySetter",
            props: {
              itemSetter: {
                componentName: "StringSetter",
                props: {
                  placeholder: "参数名称"
                }
              }
            }
          }
        }
      ],
      component: {
        isContainer: true,
        disableBehaviors: "*"
      },
      supports: false
    }
  }
});
const Page = defineComponent({
  setup(props, { attrs: attrs2, slots }) {
    return () => h(
      "div",
      { class: "lc-page", ...props, attrs: attrs2 },
      slots && slots.default && slots.default()
    );
  }
});
Object.assign(Page, {
  displayName: "Page",
  componentMetadata: {
    componentName: "Page",
    configure: {
      supports: {
        style: true,
        className: true
      },
      component: {
        isContainer: true,
        disableBehaviors: "*"
      }
    }
  }
});
var RuntimeDataSourceStatus;
(function(RuntimeDataSourceStatus2) {
  RuntimeDataSourceStatus2["Initial"] = "init";
  RuntimeDataSourceStatus2["Loading"] = "loading";
  RuntimeDataSourceStatus2["Loaded"] = "loaded";
  RuntimeDataSourceStatus2["Error"] = "error";
})(RuntimeDataSourceStatus || (RuntimeDataSourceStatus = {}));
var AssetLevel = /* @__PURE__ */ function(AssetLevel2) {
  AssetLevel2[AssetLevel2["Environment"] = 1] = "Environment";
  AssetLevel2[AssetLevel2["Library"] = 2] = "Library";
  AssetLevel2[AssetLevel2["Theme"] = 3] = "Theme";
  AssetLevel2[AssetLevel2["Runtime"] = 4] = "Runtime";
  AssetLevel2[AssetLevel2["Components"] = 5] = "Components";
  AssetLevel2[AssetLevel2["App"] = 6] = "App";
  return AssetLevel2;
}({});
[AssetLevel.Environment, AssetLevel.Library, AssetLevel.Theme, AssetLevel.Runtime, AssetLevel.Components, AssetLevel.App];
var IPublicEnumTransformStage = /* @__PURE__ */ function(IPublicEnumTransformStage2) {
  IPublicEnumTransformStage2["Render"] = "render";
  IPublicEnumTransformStage2["Serilize"] = "serilize";
  IPublicEnumTransformStage2["Save"] = "save";
  IPublicEnumTransformStage2["Clone"] = "clone";
  IPublicEnumTransformStage2["Init"] = "init";
  IPublicEnumTransformStage2["Upgrade"] = "upgrade";
  return IPublicEnumTransformStage2;
}({});
var react_production_min = {};
/*
object-assign
(c) Sindre Sorhus
@license MIT
*/
var objectAssign;
var hasRequiredObjectAssign;
function requireObjectAssign() {
  if (hasRequiredObjectAssign)
    return objectAssign;
  hasRequiredObjectAssign = 1;
  var getOwnPropertySymbols = Object.getOwnPropertySymbols;
  var hasOwnProperty = Object.prototype.hasOwnProperty;
  var propIsEnumerable = Object.prototype.propertyIsEnumerable;
  function toObject(val) {
    if (val === null || val === void 0) {
      throw new TypeError("Object.assign cannot be called with null or undefined");
    }
    return Object(val);
  }
  function shouldUseNative() {
    try {
      if (!Object.assign) {
        return false;
      }
      var test1 = new String("abc");
      test1[5] = "de";
      if (Object.getOwnPropertyNames(test1)[0] === "5") {
        return false;
      }
      var test2 = {};
      for (var i = 0; i < 10; i++) {
        test2["_" + String.fromCharCode(i)] = i;
      }
      var order2 = Object.getOwnPropertyNames(test2).map(function(n) {
        return test2[n];
      });
      if (order2.join("") !== "0123456789") {
        return false;
      }
      var test3 = {};
      "abcdefghijklmnopqrst".split("").forEach(function(letter) {
        test3[letter] = letter;
      });
      if (Object.keys(Object.assign({}, test3)).join("") !== "abcdefghijklmnopqrst") {
        return false;
      }
      return true;
    } catch (err) {
      return false;
    }
  }
  objectAssign = shouldUseNative() ? Object.assign : function(target, source) {
    var from;
    var to = toObject(target);
    var symbols;
    for (var s = 1; s < arguments.length; s++) {
      from = Object(arguments[s]);
      for (var key in from) {
        if (hasOwnProperty.call(from, key)) {
          to[key] = from[key];
        }
      }
      if (getOwnPropertySymbols) {
        symbols = getOwnPropertySymbols(from);
        for (var i = 0; i < symbols.length; i++) {
          if (propIsEnumerable.call(from, symbols[i])) {
            to[symbols[i]] = from[symbols[i]];
          }
        }
      }
    }
    return to;
  };
  return objectAssign;
}
/** @license React v16.14.0
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var hasRequiredReact_production_min;
function requireReact_production_min() {
  if (hasRequiredReact_production_min)
    return react_production_min;
  hasRequiredReact_production_min = 1;
  var l = requireObjectAssign(), n = "function" === typeof Symbol && Symbol.for, p = n ? Symbol.for("react.element") : 60103, q = n ? Symbol.for("react.portal") : 60106, r = n ? Symbol.for("react.fragment") : 60107, t = n ? Symbol.for("react.strict_mode") : 60108, u = n ? Symbol.for("react.profiler") : 60114, v = n ? Symbol.for("react.provider") : 60109, w = n ? Symbol.for("react.context") : 60110, x = n ? Symbol.for("react.forward_ref") : 60112, y = n ? Symbol.for("react.suspense") : 60113, z = n ? Symbol.for("react.memo") : 60115, A = n ? Symbol.for("react.lazy") : 60116, B = "function" === typeof Symbol && Symbol.iterator;
  function C(a) {
    for (var b = "https://reactjs.org/docs/error-decoder.html?invariant=" + a, c = 1; c < arguments.length; c++)
      b += "&args[]=" + encodeURIComponent(arguments[c]);
    return "Minified React error #" + a + "; visit " + b + " for the full message or use the non-minified dev environment for full errors and additional helpful warnings.";
  }
  var D = { isMounted: function() {
    return false;
  }, enqueueForceUpdate: function() {
  }, enqueueReplaceState: function() {
  }, enqueueSetState: function() {
  } }, E = {};
  function F(a, b, c) {
    this.props = a;
    this.context = b;
    this.refs = E;
    this.updater = c || D;
  }
  F.prototype.isReactComponent = {};
  F.prototype.setState = function(a, b) {
    if ("object" !== typeof a && "function" !== typeof a && null != a)
      throw Error(C(85));
    this.updater.enqueueSetState(this, a, b, "setState");
  };
  F.prototype.forceUpdate = function(a) {
    this.updater.enqueueForceUpdate(this, a, "forceUpdate");
  };
  function G() {
  }
  G.prototype = F.prototype;
  function H(a, b, c) {
    this.props = a;
    this.context = b;
    this.refs = E;
    this.updater = c || D;
  }
  var I = H.prototype = new G();
  I.constructor = H;
  l(I, F.prototype);
  I.isPureReactComponent = true;
  var J = { current: null }, K = Object.prototype.hasOwnProperty, L = { key: true, ref: true, __self: true, __source: true };
  function M(a, b, c) {
    var e, d = {}, g = null, k = null;
    if (null != b)
      for (e in void 0 !== b.ref && (k = b.ref), void 0 !== b.key && (g = "" + b.key), b)
        K.call(b, e) && !L.hasOwnProperty(e) && (d[e] = b[e]);
    var f = arguments.length - 2;
    if (1 === f)
      d.children = c;
    else if (1 < f) {
      for (var h2 = Array(f), m = 0; m < f; m++)
        h2[m] = arguments[m + 2];
      d.children = h2;
    }
    if (a && a.defaultProps)
      for (e in f = a.defaultProps, f)
        void 0 === d[e] && (d[e] = f[e]);
    return { $$typeof: p, type: a, key: g, ref: k, props: d, _owner: J.current };
  }
  function N(a, b) {
    return { $$typeof: p, type: a.type, key: b, ref: a.ref, props: a.props, _owner: a._owner };
  }
  function O(a) {
    return "object" === typeof a && null !== a && a.$$typeof === p;
  }
  function escape(a) {
    var b = { "=": "=0", ":": "=2" };
    return "$" + ("" + a).replace(/[=:]/g, function(a2) {
      return b[a2];
    });
  }
  var P = /\/+/g, Q = [];
  function R(a, b, c, e) {
    if (Q.length) {
      var d = Q.pop();
      d.result = a;
      d.keyPrefix = b;
      d.func = c;
      d.context = e;
      d.count = 0;
      return d;
    }
    return { result: a, keyPrefix: b, func: c, context: e, count: 0 };
  }
  function S(a) {
    a.result = null;
    a.keyPrefix = null;
    a.func = null;
    a.context = null;
    a.count = 0;
    10 > Q.length && Q.push(a);
  }
  function T(a, b, c, e) {
    var d = typeof a;
    if ("undefined" === d || "boolean" === d)
      a = null;
    var g = false;
    if (null === a)
      g = true;
    else
      switch (d) {
        case "string":
        case "number":
          g = true;
          break;
        case "object":
          switch (a.$$typeof) {
            case p:
            case q:
              g = true;
          }
      }
    if (g)
      return c(e, a, "" === b ? "." + U(a, 0) : b), 1;
    g = 0;
    b = "" === b ? "." : b + ":";
    if (Array.isArray(a))
      for (var k = 0; k < a.length; k++) {
        d = a[k];
        var f = b + U(d, k);
        g += T(d, f, c, e);
      }
    else if (null === a || "object" !== typeof a ? f = null : (f = B && a[B] || a["@@iterator"], f = "function" === typeof f ? f : null), "function" === typeof f)
      for (a = f.call(a), k = 0; !(d = a.next()).done; )
        d = d.value, f = b + U(d, k++), g += T(d, f, c, e);
    else if ("object" === d)
      throw c = "" + a, Error(C(31, "[object Object]" === c ? "object with keys {" + Object.keys(a).join(", ") + "}" : c, ""));
    return g;
  }
  function V(a, b, c) {
    return null == a ? 0 : T(a, "", b, c);
  }
  function U(a, b) {
    return "object" === typeof a && null !== a && null != a.key ? escape(a.key) : b.toString(36);
  }
  function W(a, b) {
    a.func.call(a.context, b, a.count++);
  }
  function aa(a, b, c) {
    var e = a.result, d = a.keyPrefix;
    a = a.func.call(a.context, b, a.count++);
    Array.isArray(a) ? X(a, e, c, function(a2) {
      return a2;
    }) : null != a && (O(a) && (a = N(a, d + (!a.key || b && b.key === a.key ? "" : ("" + a.key).replace(P, "$&/") + "/") + c)), e.push(a));
  }
  function X(a, b, c, e, d) {
    var g = "";
    null != c && (g = ("" + c).replace(P, "$&/") + "/");
    b = R(b, g, e, d);
    V(a, aa, b);
    S(b);
  }
  var Y = { current: null };
  function Z() {
    var a = Y.current;
    if (null === a)
      throw Error(C(321));
    return a;
  }
  var ba = { ReactCurrentDispatcher: Y, ReactCurrentBatchConfig: { suspense: null }, ReactCurrentOwner: J, IsSomeRendererActing: { current: false }, assign: l };
  react_production_min.Children = { map: function(a, b, c) {
    if (null == a)
      return a;
    var e = [];
    X(a, e, null, b, c);
    return e;
  }, forEach: function(a, b, c) {
    if (null == a)
      return a;
    b = R(null, null, b, c);
    V(a, W, b);
    S(b);
  }, count: function(a) {
    return V(a, function() {
      return null;
    }, null);
  }, toArray: function(a) {
    var b = [];
    X(a, b, null, function(a2) {
      return a2;
    });
    return b;
  }, only: function(a) {
    if (!O(a))
      throw Error(C(143));
    return a;
  } };
  react_production_min.Component = F;
  react_production_min.Fragment = r;
  react_production_min.Profiler = u;
  react_production_min.PureComponent = H;
  react_production_min.StrictMode = t;
  react_production_min.Suspense = y;
  react_production_min.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = ba;
  react_production_min.cloneElement = function(a, b, c) {
    if (null === a || void 0 === a)
      throw Error(C(267, a));
    var e = l({}, a.props), d = a.key, g = a.ref, k = a._owner;
    if (null != b) {
      void 0 !== b.ref && (g = b.ref, k = J.current);
      void 0 !== b.key && (d = "" + b.key);
      if (a.type && a.type.defaultProps)
        var f = a.type.defaultProps;
      for (h2 in b)
        K.call(b, h2) && !L.hasOwnProperty(h2) && (e[h2] = void 0 === b[h2] && void 0 !== f ? f[h2] : b[h2]);
    }
    var h2 = arguments.length - 2;
    if (1 === h2)
      e.children = c;
    else if (1 < h2) {
      f = Array(h2);
      for (var m = 0; m < h2; m++)
        f[m] = arguments[m + 2];
      e.children = f;
    }
    return {
      $$typeof: p,
      type: a.type,
      key: d,
      ref: g,
      props: e,
      _owner: k
    };
  };
  react_production_min.createContext = function(a, b) {
    void 0 === b && (b = null);
    a = { $$typeof: w, _calculateChangedBits: b, _currentValue: a, _currentValue2: a, _threadCount: 0, Provider: null, Consumer: null };
    a.Provider = { $$typeof: v, _context: a };
    return a.Consumer = a;
  };
  react_production_min.createElement = M;
  react_production_min.createFactory = function(a) {
    var b = M.bind(null, a);
    b.type = a;
    return b;
  };
  react_production_min.createRef = function() {
    return { current: null };
  };
  react_production_min.forwardRef = function(a) {
    return { $$typeof: x, render: a };
  };
  react_production_min.isValidElement = O;
  react_production_min.lazy = function(a) {
    return { $$typeof: A, _ctor: a, _status: -1, _result: null };
  };
  react_production_min.memo = function(a, b) {
    return { $$typeof: z, type: a, compare: void 0 === b ? null : b };
  };
  react_production_min.useCallback = function(a, b) {
    return Z().useCallback(a, b);
  };
  react_production_min.useContext = function(a, b) {
    return Z().useContext(a, b);
  };
  react_production_min.useDebugValue = function() {
  };
  react_production_min.useEffect = function(a, b) {
    return Z().useEffect(a, b);
  };
  react_production_min.useImperativeHandle = function(a, b, c) {
    return Z().useImperativeHandle(a, b, c);
  };
  react_production_min.useLayoutEffect = function(a, b) {
    return Z().useLayoutEffect(a, b);
  };
  react_production_min.useMemo = function(a, b) {
    return Z().useMemo(a, b);
  };
  react_production_min.useReducer = function(a, b, c) {
    return Z().useReducer(a, b, c);
  };
  react_production_min.useRef = function(a) {
    return Z().useRef(a);
  };
  react_production_min.useState = function(a) {
    return Z().useState(a);
  };
  react_production_min.version = "16.14.0";
  return react_production_min;
}
var react_development = {};
var ReactPropTypesSecret_1;
var hasRequiredReactPropTypesSecret;
function requireReactPropTypesSecret() {
  if (hasRequiredReactPropTypesSecret)
    return ReactPropTypesSecret_1;
  hasRequiredReactPropTypesSecret = 1;
  var ReactPropTypesSecret = "SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED";
  ReactPropTypesSecret_1 = ReactPropTypesSecret;
  return ReactPropTypesSecret_1;
}
var has;
var hasRequiredHas;
function requireHas() {
  if (hasRequiredHas)
    return has;
  hasRequiredHas = 1;
  has = Function.call.bind(Object.prototype.hasOwnProperty);
  return has;
}
var checkPropTypes_1;
var hasRequiredCheckPropTypes;
function requireCheckPropTypes() {
  if (hasRequiredCheckPropTypes)
    return checkPropTypes_1;
  hasRequiredCheckPropTypes = 1;
  var printWarning = function() {
  };
  if (process.env.NODE_ENV !== "production") {
    var ReactPropTypesSecret = requireReactPropTypesSecret();
    var loggedTypeFailures = {};
    var has2 = requireHas();
    printWarning = function(text) {
      var message = "Warning: " + text;
      if (typeof console !== "undefined") {
        console.error(message);
      }
      try {
        throw new Error(message);
      } catch (x) {
      }
    };
  }
  function checkPropTypes(typeSpecs, values, location, componentName, getStack) {
    if (process.env.NODE_ENV !== "production") {
      for (var typeSpecName in typeSpecs) {
        if (has2(typeSpecs, typeSpecName)) {
          var error;
          try {
            if (typeof typeSpecs[typeSpecName] !== "function") {
              var err = Error(
                (componentName || "React class") + ": " + location + " type `" + typeSpecName + "` is invalid; it must be a function, usually from the `prop-types` package, but received `" + typeof typeSpecs[typeSpecName] + "`.This often happens because of typos such as `PropTypes.function` instead of `PropTypes.func`."
              );
              err.name = "Invariant Violation";
              throw err;
            }
            error = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, ReactPropTypesSecret);
          } catch (ex) {
            error = ex;
          }
          if (error && !(error instanceof Error)) {
            printWarning(
              (componentName || "React class") + ": type specification of " + location + " `" + typeSpecName + "` is invalid; the type checker function must return `null` or an `Error` but returned a " + typeof error + ". You may have forgotten to pass an argument to the type checker creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and shape all require an argument)."
            );
          }
          if (error instanceof Error && !(error.message in loggedTypeFailures)) {
            loggedTypeFailures[error.message] = true;
            var stack = getStack ? getStack() : "";
            printWarning(
              "Failed " + location + " type: " + error.message + (stack != null ? stack : "")
            );
          }
        }
      }
    }
  }
  checkPropTypes.resetWarningCache = function() {
    if (process.env.NODE_ENV !== "production") {
      loggedTypeFailures = {};
    }
  };
  checkPropTypes_1 = checkPropTypes;
  return checkPropTypes_1;
}
/** @license React v16.14.0
 * react.development.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var hasRequiredReact_development;
function requireReact_development() {
  if (hasRequiredReact_development)
    return react_development;
  hasRequiredReact_development = 1;
  if (process.env.NODE_ENV !== "production") {
    (function() {
      var _assign = requireObjectAssign();
      var checkPropTypes = requireCheckPropTypes();
      var ReactVersion = "16.14.0";
      var hasSymbol = typeof Symbol === "function" && Symbol.for;
      var REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for("react.element") : 60103;
      var REACT_PORTAL_TYPE = hasSymbol ? Symbol.for("react.portal") : 60106;
      var REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for("react.fragment") : 60107;
      var REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for("react.strict_mode") : 60108;
      var REACT_PROFILER_TYPE = hasSymbol ? Symbol.for("react.profiler") : 60114;
      var REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for("react.provider") : 60109;
      var REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for("react.context") : 60110;
      var REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for("react.concurrent_mode") : 60111;
      var REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for("react.forward_ref") : 60112;
      var REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for("react.suspense") : 60113;
      var REACT_SUSPENSE_LIST_TYPE = hasSymbol ? Symbol.for("react.suspense_list") : 60120;
      var REACT_MEMO_TYPE = hasSymbol ? Symbol.for("react.memo") : 60115;
      var REACT_LAZY_TYPE = hasSymbol ? Symbol.for("react.lazy") : 60116;
      var REACT_BLOCK_TYPE = hasSymbol ? Symbol.for("react.block") : 60121;
      var REACT_FUNDAMENTAL_TYPE = hasSymbol ? Symbol.for("react.fundamental") : 60117;
      var REACT_RESPONDER_TYPE = hasSymbol ? Symbol.for("react.responder") : 60118;
      var REACT_SCOPE_TYPE = hasSymbol ? Symbol.for("react.scope") : 60119;
      var MAYBE_ITERATOR_SYMBOL = typeof Symbol === "function" && Symbol.iterator;
      var FAUX_ITERATOR_SYMBOL = "@@iterator";
      function getIteratorFn(maybeIterable) {
        if (maybeIterable === null || typeof maybeIterable !== "object") {
          return null;
        }
        var maybeIterator = MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL];
        if (typeof maybeIterator === "function") {
          return maybeIterator;
        }
        return null;
      }
      var ReactCurrentDispatcher = {
        /**
         * @internal
         * @type {ReactComponent}
         */
        current: null
      };
      var ReactCurrentBatchConfig = {
        suspense: null
      };
      var ReactCurrentOwner = {
        /**
         * @internal
         * @type {ReactComponent}
         */
        current: null
      };
      var BEFORE_SLASH_RE = /^(.*)[\\\/]/;
      function describeComponentFrame(name2, source, ownerName) {
        var sourceInfo = "";
        if (source) {
          var path = source.fileName;
          var fileName = path.replace(BEFORE_SLASH_RE, "");
          {
            if (/^index\./.test(fileName)) {
              var match = path.match(BEFORE_SLASH_RE);
              if (match) {
                var pathBeforeSlash = match[1];
                if (pathBeforeSlash) {
                  var folderName = pathBeforeSlash.replace(BEFORE_SLASH_RE, "");
                  fileName = folderName + "/" + fileName;
                }
              }
            }
          }
          sourceInfo = " (at " + fileName + ":" + source.lineNumber + ")";
        } else if (ownerName) {
          sourceInfo = " (created by " + ownerName + ")";
        }
        return "\n    in " + (name2 || "Unknown") + sourceInfo;
      }
      var Resolved = 1;
      function refineResolvedLazyComponent(lazyComponent) {
        return lazyComponent._status === Resolved ? lazyComponent._result : null;
      }
      function getWrappedName(outerType, innerType, wrapperName) {
        var functionName = innerType.displayName || innerType.name || "";
        return outerType.displayName || (functionName !== "" ? wrapperName + "(" + functionName + ")" : wrapperName);
      }
      function getComponentName(type) {
        if (type == null) {
          return null;
        }
        {
          if (typeof type.tag === "number") {
            error("Received an unexpected object in getComponentName(). This is likely a bug in React. Please file an issue.");
          }
        }
        if (typeof type === "function") {
          return type.displayName || type.name || null;
        }
        if (typeof type === "string") {
          return type;
        }
        switch (type) {
          case REACT_FRAGMENT_TYPE:
            return "Fragment";
          case REACT_PORTAL_TYPE:
            return "Portal";
          case REACT_PROFILER_TYPE:
            return "Profiler";
          case REACT_STRICT_MODE_TYPE:
            return "StrictMode";
          case REACT_SUSPENSE_TYPE:
            return "Suspense";
          case REACT_SUSPENSE_LIST_TYPE:
            return "SuspenseList";
        }
        if (typeof type === "object") {
          switch (type.$$typeof) {
            case REACT_CONTEXT_TYPE:
              return "Context.Consumer";
            case REACT_PROVIDER_TYPE:
              return "Context.Provider";
            case REACT_FORWARD_REF_TYPE:
              return getWrappedName(type, type.render, "ForwardRef");
            case REACT_MEMO_TYPE:
              return getComponentName(type.type);
            case REACT_BLOCK_TYPE:
              return getComponentName(type.render);
            case REACT_LAZY_TYPE: {
              var thenable = type;
              var resolvedThenable = refineResolvedLazyComponent(thenable);
              if (resolvedThenable) {
                return getComponentName(resolvedThenable);
              }
              break;
            }
          }
        }
        return null;
      }
      var ReactDebugCurrentFrame = {};
      var currentlyValidatingElement = null;
      function setCurrentlyValidatingElement(element) {
        {
          currentlyValidatingElement = element;
        }
      }
      {
        ReactDebugCurrentFrame.getCurrentStack = null;
        ReactDebugCurrentFrame.getStackAddendum = function() {
          var stack = "";
          if (currentlyValidatingElement) {
            var name2 = getComponentName(currentlyValidatingElement.type);
            var owner = currentlyValidatingElement._owner;
            stack += describeComponentFrame(name2, currentlyValidatingElement._source, owner && getComponentName(owner.type));
          }
          var impl = ReactDebugCurrentFrame.getCurrentStack;
          if (impl) {
            stack += impl() || "";
          }
          return stack;
        };
      }
      var IsSomeRendererActing = {
        current: false
      };
      var ReactSharedInternals = {
        ReactCurrentDispatcher,
        ReactCurrentBatchConfig,
        ReactCurrentOwner,
        IsSomeRendererActing,
        // Used by renderers to avoid bundling object-assign twice in UMD bundles:
        assign: _assign
      };
      {
        _assign(ReactSharedInternals, {
          // These should not be included in production.
          ReactDebugCurrentFrame,
          // Shim for React DOM 16.0.0 which still destructured (but not used) this.
          // TODO: remove in React 17.0.
          ReactComponentTreeHook: {}
        });
      }
      function warn(format) {
        {
          for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
          }
          printWarning("warn", format, args);
        }
      }
      function error(format) {
        {
          for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
            args[_key2 - 1] = arguments[_key2];
          }
          printWarning("error", format, args);
        }
      }
      function printWarning(level, format, args) {
        {
          var hasExistingStack = args.length > 0 && typeof args[args.length - 1] === "string" && args[args.length - 1].indexOf("\n    in") === 0;
          if (!hasExistingStack) {
            var ReactDebugCurrentFrame2 = ReactSharedInternals.ReactDebugCurrentFrame;
            var stack = ReactDebugCurrentFrame2.getStackAddendum();
            if (stack !== "") {
              format += "%s";
              args = args.concat([stack]);
            }
          }
          var argsWithFormat = args.map(function(item) {
            return "" + item;
          });
          argsWithFormat.unshift("Warning: " + format);
          Function.prototype.apply.call(console[level], console, argsWithFormat);
          try {
            var argIndex = 0;
            var message = "Warning: " + format.replace(/%s/g, function() {
              return args[argIndex++];
            });
            throw new Error(message);
          } catch (x) {
          }
        }
      }
      var didWarnStateUpdateForUnmountedComponent = {};
      function warnNoop(publicInstance, callerName) {
        {
          var _constructor = publicInstance.constructor;
          var componentName = _constructor && (_constructor.displayName || _constructor.name) || "ReactClass";
          var warningKey = componentName + "." + callerName;
          if (didWarnStateUpdateForUnmountedComponent[warningKey]) {
            return;
          }
          error("Can't call %s on a component that is not yet mounted. This is a no-op, but it might indicate a bug in your application. Instead, assign to `this.state` directly or define a `state = {};` class property with the desired state in the %s component.", callerName, componentName);
          didWarnStateUpdateForUnmountedComponent[warningKey] = true;
        }
      }
      var ReactNoopUpdateQueue = {
        /**
         * Checks whether or not this composite component is mounted.
         * @param {ReactClass} publicInstance The instance we want to test.
         * @return {boolean} True if mounted, false otherwise.
         * @protected
         * @final
         */
        isMounted: function(publicInstance) {
          return false;
        },
        /**
         * Forces an update. This should only be invoked when it is known with
         * certainty that we are **not** in a DOM transaction.
         *
         * You may want to call this when you know that some deeper aspect of the
         * component's state has changed but `setState` was not called.
         *
         * This will not invoke `shouldComponentUpdate`, but it will invoke
         * `componentWillUpdate` and `componentDidUpdate`.
         *
         * @param {ReactClass} publicInstance The instance that should rerender.
         * @param {?function} callback Called after component is updated.
         * @param {?string} callerName name of the calling function in the public API.
         * @internal
         */
        enqueueForceUpdate: function(publicInstance, callback, callerName) {
          warnNoop(publicInstance, "forceUpdate");
        },
        /**
         * Replaces all of the state. Always use this or `setState` to mutate state.
         * You should treat `this.state` as immutable.
         *
         * There is no guarantee that `this.state` will be immediately updated, so
         * accessing `this.state` after calling this method may return the old value.
         *
         * @param {ReactClass} publicInstance The instance that should rerender.
         * @param {object} completeState Next state.
         * @param {?function} callback Called after component is updated.
         * @param {?string} callerName name of the calling function in the public API.
         * @internal
         */
        enqueueReplaceState: function(publicInstance, completeState, callback, callerName) {
          warnNoop(publicInstance, "replaceState");
        },
        /**
         * Sets a subset of the state. This only exists because _pendingState is
         * internal. This provides a merging strategy that is not available to deep
         * properties which is confusing. TODO: Expose pendingState or don't use it
         * during the merge.
         *
         * @param {ReactClass} publicInstance The instance that should rerender.
         * @param {object} partialState Next partial state to be merged with state.
         * @param {?function} callback Called after component is updated.
         * @param {?string} Name of the calling function in the public API.
         * @internal
         */
        enqueueSetState: function(publicInstance, partialState, callback, callerName) {
          warnNoop(publicInstance, "setState");
        }
      };
      var emptyObject = {};
      {
        Object.freeze(emptyObject);
      }
      function Component(props, context, updater) {
        this.props = props;
        this.context = context;
        this.refs = emptyObject;
        this.updater = updater || ReactNoopUpdateQueue;
      }
      Component.prototype.isReactComponent = {};
      Component.prototype.setState = function(partialState, callback) {
        if (!(typeof partialState === "object" || typeof partialState === "function" || partialState == null)) {
          {
            throw Error("setState(...): takes an object of state variables to update or a function which returns an object of state variables.");
          }
        }
        this.updater.enqueueSetState(this, partialState, callback, "setState");
      };
      Component.prototype.forceUpdate = function(callback) {
        this.updater.enqueueForceUpdate(this, callback, "forceUpdate");
      };
      {
        var deprecatedAPIs = {
          isMounted: ["isMounted", "Instead, make sure to clean up subscriptions and pending requests in componentWillUnmount to prevent memory leaks."],
          replaceState: ["replaceState", "Refactor your code to use setState instead (see https://github.com/facebook/react/issues/3236)."]
        };
        var defineDeprecationWarning = function(methodName, info) {
          Object.defineProperty(Component.prototype, methodName, {
            get: function() {
              warn("%s(...) is deprecated in plain JavaScript React classes. %s", info[0], info[1]);
              return void 0;
            }
          });
        };
        for (var fnName in deprecatedAPIs) {
          if (deprecatedAPIs.hasOwnProperty(fnName)) {
            defineDeprecationWarning(fnName, deprecatedAPIs[fnName]);
          }
        }
      }
      function ComponentDummy() {
      }
      ComponentDummy.prototype = Component.prototype;
      function PureComponent(props, context, updater) {
        this.props = props;
        this.context = context;
        this.refs = emptyObject;
        this.updater = updater || ReactNoopUpdateQueue;
      }
      var pureComponentPrototype = PureComponent.prototype = new ComponentDummy();
      pureComponentPrototype.constructor = PureComponent;
      _assign(pureComponentPrototype, Component.prototype);
      pureComponentPrototype.isPureReactComponent = true;
      function createRef() {
        var refObject = {
          current: null
        };
        {
          Object.seal(refObject);
        }
        return refObject;
      }
      var hasOwnProperty = Object.prototype.hasOwnProperty;
      var RESERVED_PROPS = {
        key: true,
        ref: true,
        __self: true,
        __source: true
      };
      var specialPropKeyWarningShown, specialPropRefWarningShown, didWarnAboutStringRefs;
      {
        didWarnAboutStringRefs = {};
      }
      function hasValidRef(config3) {
        {
          if (hasOwnProperty.call(config3, "ref")) {
            var getter = Object.getOwnPropertyDescriptor(config3, "ref").get;
            if (getter && getter.isReactWarning) {
              return false;
            }
          }
        }
        return config3.ref !== void 0;
      }
      function hasValidKey(config3) {
        {
          if (hasOwnProperty.call(config3, "key")) {
            var getter = Object.getOwnPropertyDescriptor(config3, "key").get;
            if (getter && getter.isReactWarning) {
              return false;
            }
          }
        }
        return config3.key !== void 0;
      }
      function defineKeyPropWarningGetter(props, displayName) {
        var warnAboutAccessingKey = function() {
          {
            if (!specialPropKeyWarningShown) {
              specialPropKeyWarningShown = true;
              error("%s: `key` is not a prop. Trying to access it will result in `undefined` being returned. If you need to access the same value within the child component, you should pass it as a different prop. (https://fb.me/react-special-props)", displayName);
            }
          }
        };
        warnAboutAccessingKey.isReactWarning = true;
        Object.defineProperty(props, "key", {
          get: warnAboutAccessingKey,
          configurable: true
        });
      }
      function defineRefPropWarningGetter(props, displayName) {
        var warnAboutAccessingRef = function() {
          {
            if (!specialPropRefWarningShown) {
              specialPropRefWarningShown = true;
              error("%s: `ref` is not a prop. Trying to access it will result in `undefined` being returned. If you need to access the same value within the child component, you should pass it as a different prop. (https://fb.me/react-special-props)", displayName);
            }
          }
        };
        warnAboutAccessingRef.isReactWarning = true;
        Object.defineProperty(props, "ref", {
          get: warnAboutAccessingRef,
          configurable: true
        });
      }
      function warnIfStringRefCannotBeAutoConverted(config3) {
        {
          if (typeof config3.ref === "string" && ReactCurrentOwner.current && config3.__self && ReactCurrentOwner.current.stateNode !== config3.__self) {
            var componentName = getComponentName(ReactCurrentOwner.current.type);
            if (!didWarnAboutStringRefs[componentName]) {
              error('Component "%s" contains the string ref "%s". Support for string refs will be removed in a future major release. This case cannot be automatically converted to an arrow function. We ask you to manually fix this case by using useRef() or createRef() instead. Learn more about using refs safely here: https://fb.me/react-strict-mode-string-ref', getComponentName(ReactCurrentOwner.current.type), config3.ref);
              didWarnAboutStringRefs[componentName] = true;
            }
          }
        }
      }
      var ReactElement = function(type, key, ref2, self, source, owner, props) {
        var element = {
          // This tag allows us to uniquely identify this as a React Element
          $$typeof: REACT_ELEMENT_TYPE,
          // Built-in properties that belong on the element
          type,
          key,
          ref: ref2,
          props,
          // Record the component responsible for creating this element.
          _owner: owner
        };
        {
          element._store = {};
          Object.defineProperty(element._store, "validated", {
            configurable: false,
            enumerable: false,
            writable: true,
            value: false
          });
          Object.defineProperty(element, "_self", {
            configurable: false,
            enumerable: false,
            writable: false,
            value: self
          });
          Object.defineProperty(element, "_source", {
            configurable: false,
            enumerable: false,
            writable: false,
            value: source
          });
          if (Object.freeze) {
            Object.freeze(element.props);
            Object.freeze(element);
          }
        }
        return element;
      };
      function createElement(type, config3, children) {
        var propName;
        var props = {};
        var key = null;
        var ref2 = null;
        var self = null;
        var source = null;
        if (config3 != null) {
          if (hasValidRef(config3)) {
            ref2 = config3.ref;
            {
              warnIfStringRefCannotBeAutoConverted(config3);
            }
          }
          if (hasValidKey(config3)) {
            key = "" + config3.key;
          }
          self = config3.__self === void 0 ? null : config3.__self;
          source = config3.__source === void 0 ? null : config3.__source;
          for (propName in config3) {
            if (hasOwnProperty.call(config3, propName) && !RESERVED_PROPS.hasOwnProperty(propName)) {
              props[propName] = config3[propName];
            }
          }
        }
        var childrenLength = arguments.length - 2;
        if (childrenLength === 1) {
          props.children = children;
        } else if (childrenLength > 1) {
          var childArray = Array(childrenLength);
          for (var i = 0; i < childrenLength; i++) {
            childArray[i] = arguments[i + 2];
          }
          {
            if (Object.freeze) {
              Object.freeze(childArray);
            }
          }
          props.children = childArray;
        }
        if (type && type.defaultProps) {
          var defaultProps = type.defaultProps;
          for (propName in defaultProps) {
            if (props[propName] === void 0) {
              props[propName] = defaultProps[propName];
            }
          }
        }
        {
          if (key || ref2) {
            var displayName = typeof type === "function" ? type.displayName || type.name || "Unknown" : type;
            if (key) {
              defineKeyPropWarningGetter(props, displayName);
            }
            if (ref2) {
              defineRefPropWarningGetter(props, displayName);
            }
          }
        }
        return ReactElement(type, key, ref2, self, source, ReactCurrentOwner.current, props);
      }
      function cloneAndReplaceKey(oldElement, newKey) {
        var newElement = ReactElement(oldElement.type, newKey, oldElement.ref, oldElement._self, oldElement._source, oldElement._owner, oldElement.props);
        return newElement;
      }
      function cloneElement(element, config3, children) {
        if (!!(element === null || element === void 0)) {
          {
            throw Error("React.cloneElement(...): The argument must be a React element, but you passed " + element + ".");
          }
        }
        var propName;
        var props = _assign({}, element.props);
        var key = element.key;
        var ref2 = element.ref;
        var self = element._self;
        var source = element._source;
        var owner = element._owner;
        if (config3 != null) {
          if (hasValidRef(config3)) {
            ref2 = config3.ref;
            owner = ReactCurrentOwner.current;
          }
          if (hasValidKey(config3)) {
            key = "" + config3.key;
          }
          var defaultProps;
          if (element.type && element.type.defaultProps) {
            defaultProps = element.type.defaultProps;
          }
          for (propName in config3) {
            if (hasOwnProperty.call(config3, propName) && !RESERVED_PROPS.hasOwnProperty(propName)) {
              if (config3[propName] === void 0 && defaultProps !== void 0) {
                props[propName] = defaultProps[propName];
              } else {
                props[propName] = config3[propName];
              }
            }
          }
        }
        var childrenLength = arguments.length - 2;
        if (childrenLength === 1) {
          props.children = children;
        } else if (childrenLength > 1) {
          var childArray = Array(childrenLength);
          for (var i = 0; i < childrenLength; i++) {
            childArray[i] = arguments[i + 2];
          }
          props.children = childArray;
        }
        return ReactElement(element.type, key, ref2, self, source, owner, props);
      }
      function isValidElement2(object) {
        return typeof object === "object" && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
      }
      var SEPARATOR = ".";
      var SUBSEPARATOR = ":";
      function escape(key) {
        var escapeRegex = /[=:]/g;
        var escaperLookup = {
          "=": "=0",
          ":": "=2"
        };
        var escapedString = ("" + key).replace(escapeRegex, function(match) {
          return escaperLookup[match];
        });
        return "$" + escapedString;
      }
      var didWarnAboutMaps = false;
      var userProvidedKeyEscapeRegex = /\/+/g;
      function escapeUserProvidedKey(text) {
        return ("" + text).replace(userProvidedKeyEscapeRegex, "$&/");
      }
      var POOL_SIZE = 10;
      var traverseContextPool = [];
      function getPooledTraverseContext(mapResult, keyPrefix, mapFunction, mapContext) {
        if (traverseContextPool.length) {
          var traverseContext = traverseContextPool.pop();
          traverseContext.result = mapResult;
          traverseContext.keyPrefix = keyPrefix;
          traverseContext.func = mapFunction;
          traverseContext.context = mapContext;
          traverseContext.count = 0;
          return traverseContext;
        } else {
          return {
            result: mapResult,
            keyPrefix,
            func: mapFunction,
            context: mapContext,
            count: 0
          };
        }
      }
      function releaseTraverseContext(traverseContext) {
        traverseContext.result = null;
        traverseContext.keyPrefix = null;
        traverseContext.func = null;
        traverseContext.context = null;
        traverseContext.count = 0;
        if (traverseContextPool.length < POOL_SIZE) {
          traverseContextPool.push(traverseContext);
        }
      }
      function traverseAllChildrenImpl(children, nameSoFar, callback, traverseContext) {
        var type = typeof children;
        if (type === "undefined" || type === "boolean") {
          children = null;
        }
        var invokeCallback = false;
        if (children === null) {
          invokeCallback = true;
        } else {
          switch (type) {
            case "string":
            case "number":
              invokeCallback = true;
              break;
            case "object":
              switch (children.$$typeof) {
                case REACT_ELEMENT_TYPE:
                case REACT_PORTAL_TYPE:
                  invokeCallback = true;
              }
          }
        }
        if (invokeCallback) {
          callback(
            traverseContext,
            children,
            // If it's the only child, treat the name as if it was wrapped in an array
            // so that it's consistent if the number of children grows.
            nameSoFar === "" ? SEPARATOR + getComponentKey(children, 0) : nameSoFar
          );
          return 1;
        }
        var child;
        var nextName;
        var subtreeCount = 0;
        var nextNamePrefix = nameSoFar === "" ? SEPARATOR : nameSoFar + SUBSEPARATOR;
        if (Array.isArray(children)) {
          for (var i = 0; i < children.length; i++) {
            child = children[i];
            nextName = nextNamePrefix + getComponentKey(child, i);
            subtreeCount += traverseAllChildrenImpl(child, nextName, callback, traverseContext);
          }
        } else {
          var iteratorFn = getIteratorFn(children);
          if (typeof iteratorFn === "function") {
            {
              if (iteratorFn === children.entries) {
                if (!didWarnAboutMaps) {
                  warn("Using Maps as children is deprecated and will be removed in a future major release. Consider converting children to an array of keyed ReactElements instead.");
                }
                didWarnAboutMaps = true;
              }
            }
            var iterator = iteratorFn.call(children);
            var step;
            var ii = 0;
            while (!(step = iterator.next()).done) {
              child = step.value;
              nextName = nextNamePrefix + getComponentKey(child, ii++);
              subtreeCount += traverseAllChildrenImpl(child, nextName, callback, traverseContext);
            }
          } else if (type === "object") {
            var addendum = "";
            {
              addendum = " If you meant to render a collection of children, use an array instead." + ReactDebugCurrentFrame.getStackAddendum();
            }
            var childrenString = "" + children;
            {
              {
                throw Error("Objects are not valid as a React child (found: " + (childrenString === "[object Object]" ? "object with keys {" + Object.keys(children).join(", ") + "}" : childrenString) + ")." + addendum);
              }
            }
          }
        }
        return subtreeCount;
      }
      function traverseAllChildren(children, callback, traverseContext) {
        if (children == null) {
          return 0;
        }
        return traverseAllChildrenImpl(children, "", callback, traverseContext);
      }
      function getComponentKey(component, index2) {
        if (typeof component === "object" && component !== null && component.key != null) {
          return escape(component.key);
        }
        return index2.toString(36);
      }
      function forEachSingleChild(bookKeeping, child, name2) {
        var func = bookKeeping.func, context = bookKeeping.context;
        func.call(context, child, bookKeeping.count++);
      }
      function forEachChildren(children, forEachFunc, forEachContext) {
        if (children == null) {
          return children;
        }
        var traverseContext = getPooledTraverseContext(null, null, forEachFunc, forEachContext);
        traverseAllChildren(children, forEachSingleChild, traverseContext);
        releaseTraverseContext(traverseContext);
      }
      function mapSingleChildIntoContext(bookKeeping, child, childKey) {
        var result = bookKeeping.result, keyPrefix = bookKeeping.keyPrefix, func = bookKeeping.func, context = bookKeeping.context;
        var mappedChild = func.call(context, child, bookKeeping.count++);
        if (Array.isArray(mappedChild)) {
          mapIntoWithKeyPrefixInternal(mappedChild, result, childKey, function(c) {
            return c;
          });
        } else if (mappedChild != null) {
          if (isValidElement2(mappedChild)) {
            mappedChild = cloneAndReplaceKey(
              mappedChild,
              // Keep both the (mapped) and old keys if they differ, just as
              // traverseAllChildren used to do for objects as children
              keyPrefix + (mappedChild.key && (!child || child.key !== mappedChild.key) ? escapeUserProvidedKey(mappedChild.key) + "/" : "") + childKey
            );
          }
          result.push(mappedChild);
        }
      }
      function mapIntoWithKeyPrefixInternal(children, array, prefix, func, context) {
        var escapedPrefix = "";
        if (prefix != null) {
          escapedPrefix = escapeUserProvidedKey(prefix) + "/";
        }
        var traverseContext = getPooledTraverseContext(array, escapedPrefix, func, context);
        traverseAllChildren(children, mapSingleChildIntoContext, traverseContext);
        releaseTraverseContext(traverseContext);
      }
      function mapChildren(children, func, context) {
        if (children == null) {
          return children;
        }
        var result = [];
        mapIntoWithKeyPrefixInternal(children, result, null, func, context);
        return result;
      }
      function countChildren(children) {
        return traverseAllChildren(children, function() {
          return null;
        }, null);
      }
      function toArray(children) {
        var result = [];
        mapIntoWithKeyPrefixInternal(children, result, null, function(child) {
          return child;
        });
        return result;
      }
      function onlyChild(children) {
        if (!isValidElement2(children)) {
          {
            throw Error("React.Children.only expected to receive a single React element child.");
          }
        }
        return children;
      }
      function createContext(defaultValue, calculateChangedBits) {
        if (calculateChangedBits === void 0) {
          calculateChangedBits = null;
        } else {
          {
            if (calculateChangedBits !== null && typeof calculateChangedBits !== "function") {
              error("createContext: Expected the optional second argument to be a function. Instead received: %s", calculateChangedBits);
            }
          }
        }
        var context = {
          $$typeof: REACT_CONTEXT_TYPE,
          _calculateChangedBits: calculateChangedBits,
          // As a workaround to support multiple concurrent renderers, we categorize
          // some renderers as primary and others as secondary. We only expect
          // there to be two concurrent renderers at most: React Native (primary) and
          // Fabric (secondary); React DOM (primary) and React ART (secondary).
          // Secondary renderers store their context values on separate fields.
          _currentValue: defaultValue,
          _currentValue2: defaultValue,
          // Used to track how many concurrent renderers this context currently
          // supports within in a single renderer. Such as parallel server rendering.
          _threadCount: 0,
          // These are circular
          Provider: null,
          Consumer: null
        };
        context.Provider = {
          $$typeof: REACT_PROVIDER_TYPE,
          _context: context
        };
        var hasWarnedAboutUsingNestedContextConsumers = false;
        var hasWarnedAboutUsingConsumerProvider = false;
        {
          var Consumer = {
            $$typeof: REACT_CONTEXT_TYPE,
            _context: context,
            _calculateChangedBits: context._calculateChangedBits
          };
          Object.defineProperties(Consumer, {
            Provider: {
              get: function() {
                if (!hasWarnedAboutUsingConsumerProvider) {
                  hasWarnedAboutUsingConsumerProvider = true;
                  error("Rendering <Context.Consumer.Provider> is not supported and will be removed in a future major release. Did you mean to render <Context.Provider> instead?");
                }
                return context.Provider;
              },
              set: function(_Provider) {
                context.Provider = _Provider;
              }
            },
            _currentValue: {
              get: function() {
                return context._currentValue;
              },
              set: function(_currentValue) {
                context._currentValue = _currentValue;
              }
            },
            _currentValue2: {
              get: function() {
                return context._currentValue2;
              },
              set: function(_currentValue2) {
                context._currentValue2 = _currentValue2;
              }
            },
            _threadCount: {
              get: function() {
                return context._threadCount;
              },
              set: function(_threadCount) {
                context._threadCount = _threadCount;
              }
            },
            Consumer: {
              get: function() {
                if (!hasWarnedAboutUsingNestedContextConsumers) {
                  hasWarnedAboutUsingNestedContextConsumers = true;
                  error("Rendering <Context.Consumer.Consumer> is not supported and will be removed in a future major release. Did you mean to render <Context.Consumer> instead?");
                }
                return context.Consumer;
              }
            }
          });
          context.Consumer = Consumer;
        }
        {
          context._currentRenderer = null;
          context._currentRenderer2 = null;
        }
        return context;
      }
      function lazy(ctor) {
        var lazyType = {
          $$typeof: REACT_LAZY_TYPE,
          _ctor: ctor,
          // React uses these fields to store the result.
          _status: -1,
          _result: null
        };
        {
          var defaultProps;
          var propTypes;
          Object.defineProperties(lazyType, {
            defaultProps: {
              configurable: true,
              get: function() {
                return defaultProps;
              },
              set: function(newDefaultProps) {
                error("React.lazy(...): It is not supported to assign `defaultProps` to a lazy component import. Either specify them where the component is defined, or create a wrapping component around it.");
                defaultProps = newDefaultProps;
                Object.defineProperty(lazyType, "defaultProps", {
                  enumerable: true
                });
              }
            },
            propTypes: {
              configurable: true,
              get: function() {
                return propTypes;
              },
              set: function(newPropTypes) {
                error("React.lazy(...): It is not supported to assign `propTypes` to a lazy component import. Either specify them where the component is defined, or create a wrapping component around it.");
                propTypes = newPropTypes;
                Object.defineProperty(lazyType, "propTypes", {
                  enumerable: true
                });
              }
            }
          });
        }
        return lazyType;
      }
      function forwardRef(render) {
        {
          if (render != null && render.$$typeof === REACT_MEMO_TYPE) {
            error("forwardRef requires a render function but received a `memo` component. Instead of forwardRef(memo(...)), use memo(forwardRef(...)).");
          } else if (typeof render !== "function") {
            error("forwardRef requires a render function but was given %s.", render === null ? "null" : typeof render);
          } else {
            if (render.length !== 0 && render.length !== 2) {
              error("forwardRef render functions accept exactly two parameters: props and ref. %s", render.length === 1 ? "Did you forget to use the ref parameter?" : "Any additional parameter will be undefined.");
            }
          }
          if (render != null) {
            if (render.defaultProps != null || render.propTypes != null) {
              error("forwardRef render functions do not support propTypes or defaultProps. Did you accidentally pass a React component?");
            }
          }
        }
        return {
          $$typeof: REACT_FORWARD_REF_TYPE,
          render
        };
      }
      function isValidElementType(type) {
        return typeof type === "string" || typeof type === "function" || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.
        type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || typeof type === "object" && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_FUNDAMENTAL_TYPE || type.$$typeof === REACT_RESPONDER_TYPE || type.$$typeof === REACT_SCOPE_TYPE || type.$$typeof === REACT_BLOCK_TYPE);
      }
      function memo(type, compare) {
        {
          if (!isValidElementType(type)) {
            error("memo: The first argument must be a component. Instead received: %s", type === null ? "null" : typeof type);
          }
        }
        return {
          $$typeof: REACT_MEMO_TYPE,
          type,
          compare: compare === void 0 ? null : compare
        };
      }
      function resolveDispatcher() {
        var dispatcher = ReactCurrentDispatcher.current;
        if (!(dispatcher !== null)) {
          {
            throw Error("Invalid hook call. Hooks can only be called inside of the body of a function component. This could happen for one of the following reasons:\n1. You might have mismatching versions of React and the renderer (such as React DOM)\n2. You might be breaking the Rules of Hooks\n3. You might have more than one copy of React in the same app\nSee https://fb.me/react-invalid-hook-call for tips about how to debug and fix this problem.");
          }
        }
        return dispatcher;
      }
      function useContext(Context, unstable_observedBits) {
        var dispatcher = resolveDispatcher();
        {
          if (unstable_observedBits !== void 0) {
            error("useContext() second argument is reserved for future use in React. Passing it is not supported. You passed: %s.%s", unstable_observedBits, typeof unstable_observedBits === "number" && Array.isArray(arguments[2]) ? "\n\nDid you call array.map(useContext)? Calling Hooks inside a loop is not supported. Learn more at https://fb.me/rules-of-hooks" : "");
          }
          if (Context._context !== void 0) {
            var realContext = Context._context;
            if (realContext.Consumer === Context) {
              error("Calling useContext(Context.Consumer) is not supported, may cause bugs, and will be removed in a future major release. Did you mean to call useContext(Context) instead?");
            } else if (realContext.Provider === Context) {
              error("Calling useContext(Context.Provider) is not supported. Did you mean to call useContext(Context) instead?");
            }
          }
        }
        return dispatcher.useContext(Context, unstable_observedBits);
      }
      function useState(initialState) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useState(initialState);
      }
      function useReducer(reducer, initialArg, init) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useReducer(reducer, initialArg, init);
      }
      function useRef(initialValue) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useRef(initialValue);
      }
      function useEffect(create, deps) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useEffect(create, deps);
      }
      function useLayoutEffect(create, deps) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useLayoutEffect(create, deps);
      }
      function useCallback(callback, deps) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useCallback(callback, deps);
      }
      function useMemo(create, deps) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useMemo(create, deps);
      }
      function useImperativeHandle(ref2, create, deps) {
        var dispatcher = resolveDispatcher();
        return dispatcher.useImperativeHandle(ref2, create, deps);
      }
      function useDebugValue(value, formatterFn) {
        {
          var dispatcher = resolveDispatcher();
          return dispatcher.useDebugValue(value, formatterFn);
        }
      }
      var propTypesMisspellWarningShown;
      {
        propTypesMisspellWarningShown = false;
      }
      function getDeclarationErrorAddendum() {
        if (ReactCurrentOwner.current) {
          var name2 = getComponentName(ReactCurrentOwner.current.type);
          if (name2) {
            return "\n\nCheck the render method of `" + name2 + "`.";
          }
        }
        return "";
      }
      function getSourceInfoErrorAddendum(source) {
        if (source !== void 0) {
          var fileName = source.fileName.replace(/^.*[\\\/]/, "");
          var lineNumber = source.lineNumber;
          return "\n\nCheck your code at " + fileName + ":" + lineNumber + ".";
        }
        return "";
      }
      function getSourceInfoErrorAddendumForProps(elementProps) {
        if (elementProps !== null && elementProps !== void 0) {
          return getSourceInfoErrorAddendum(elementProps.__source);
        }
        return "";
      }
      var ownerHasKeyUseWarning = {};
      function getCurrentComponentErrorInfo(parentType) {
        var info = getDeclarationErrorAddendum();
        if (!info) {
          var parentName = typeof parentType === "string" ? parentType : parentType.displayName || parentType.name;
          if (parentName) {
            info = "\n\nCheck the top-level render call using <" + parentName + ">.";
          }
        }
        return info;
      }
      function validateExplicitKey(element, parentType) {
        if (!element._store || element._store.validated || element.key != null) {
          return;
        }
        element._store.validated = true;
        var currentComponentErrorInfo = getCurrentComponentErrorInfo(parentType);
        if (ownerHasKeyUseWarning[currentComponentErrorInfo]) {
          return;
        }
        ownerHasKeyUseWarning[currentComponentErrorInfo] = true;
        var childOwner = "";
        if (element && element._owner && element._owner !== ReactCurrentOwner.current) {
          childOwner = " It was passed a child from " + getComponentName(element._owner.type) + ".";
        }
        setCurrentlyValidatingElement(element);
        {
          error('Each child in a list should have a unique "key" prop.%s%s See https://fb.me/react-warning-keys for more information.', currentComponentErrorInfo, childOwner);
        }
        setCurrentlyValidatingElement(null);
      }
      function validateChildKeys(node, parentType) {
        if (typeof node !== "object") {
          return;
        }
        if (Array.isArray(node)) {
          for (var i = 0; i < node.length; i++) {
            var child = node[i];
            if (isValidElement2(child)) {
              validateExplicitKey(child, parentType);
            }
          }
        } else if (isValidElement2(node)) {
          if (node._store) {
            node._store.validated = true;
          }
        } else if (node) {
          var iteratorFn = getIteratorFn(node);
          if (typeof iteratorFn === "function") {
            if (iteratorFn !== node.entries) {
              var iterator = iteratorFn.call(node);
              var step;
              while (!(step = iterator.next()).done) {
                if (isValidElement2(step.value)) {
                  validateExplicitKey(step.value, parentType);
                }
              }
            }
          }
        }
      }
      function validatePropTypes(element) {
        {
          var type = element.type;
          if (type === null || type === void 0 || typeof type === "string") {
            return;
          }
          var name2 = getComponentName(type);
          var propTypes;
          if (typeof type === "function") {
            propTypes = type.propTypes;
          } else if (typeof type === "object" && (type.$$typeof === REACT_FORWARD_REF_TYPE || // Note: Memo only checks outer props here.
          // Inner props are checked in the reconciler.
          type.$$typeof === REACT_MEMO_TYPE)) {
            propTypes = type.propTypes;
          } else {
            return;
          }
          if (propTypes) {
            setCurrentlyValidatingElement(element);
            checkPropTypes(propTypes, element.props, "prop", name2, ReactDebugCurrentFrame.getStackAddendum);
            setCurrentlyValidatingElement(null);
          } else if (type.PropTypes !== void 0 && !propTypesMisspellWarningShown) {
            propTypesMisspellWarningShown = true;
            error("Component %s declared `PropTypes` instead of `propTypes`. Did you misspell the property assignment?", name2 || "Unknown");
          }
          if (typeof type.getDefaultProps === "function" && !type.getDefaultProps.isReactClassApproved) {
            error("getDefaultProps is only used on classic React.createClass definitions. Use a static property named `defaultProps` instead.");
          }
        }
      }
      function validateFragmentProps(fragment) {
        {
          setCurrentlyValidatingElement(fragment);
          var keys = Object.keys(fragment.props);
          for (var i = 0; i < keys.length; i++) {
            var key = keys[i];
            if (key !== "children" && key !== "key") {
              error("Invalid prop `%s` supplied to `React.Fragment`. React.Fragment can only have `key` and `children` props.", key);
              break;
            }
          }
          if (fragment.ref !== null) {
            error("Invalid attribute `ref` supplied to `React.Fragment`.");
          }
          setCurrentlyValidatingElement(null);
        }
      }
      function createElementWithValidation(type, props, children) {
        var validType = isValidElementType(type);
        if (!validType) {
          var info = "";
          if (type === void 0 || typeof type === "object" && type !== null && Object.keys(type).length === 0) {
            info += " You likely forgot to export your component from the file it's defined in, or you might have mixed up default and named imports.";
          }
          var sourceInfo = getSourceInfoErrorAddendumForProps(props);
          if (sourceInfo) {
            info += sourceInfo;
          } else {
            info += getDeclarationErrorAddendum();
          }
          var typeString;
          if (type === null) {
            typeString = "null";
          } else if (Array.isArray(type)) {
            typeString = "array";
          } else if (type !== void 0 && type.$$typeof === REACT_ELEMENT_TYPE) {
            typeString = "<" + (getComponentName(type.type) || "Unknown") + " />";
            info = " Did you accidentally export a JSX literal instead of a component?";
          } else {
            typeString = typeof type;
          }
          {
            error("React.createElement: type is invalid -- expected a string (for built-in components) or a class/function (for composite components) but got: %s.%s", typeString, info);
          }
        }
        var element = createElement.apply(this, arguments);
        if (element == null) {
          return element;
        }
        if (validType) {
          for (var i = 2; i < arguments.length; i++) {
            validateChildKeys(arguments[i], type);
          }
        }
        if (type === REACT_FRAGMENT_TYPE) {
          validateFragmentProps(element);
        } else {
          validatePropTypes(element);
        }
        return element;
      }
      var didWarnAboutDeprecatedCreateFactory = false;
      function createFactoryWithValidation(type) {
        var validatedFactory = createElementWithValidation.bind(null, type);
        validatedFactory.type = type;
        {
          if (!didWarnAboutDeprecatedCreateFactory) {
            didWarnAboutDeprecatedCreateFactory = true;
            warn("React.createFactory() is deprecated and will be removed in a future major release. Consider using JSX or use React.createElement() directly instead.");
          }
          Object.defineProperty(validatedFactory, "type", {
            enumerable: false,
            get: function() {
              warn("Factory.type is deprecated. Access the class directly before passing it to createFactory.");
              Object.defineProperty(this, "type", {
                value: type
              });
              return type;
            }
          });
        }
        return validatedFactory;
      }
      function cloneElementWithValidation(element, props, children) {
        var newElement = cloneElement.apply(this, arguments);
        for (var i = 2; i < arguments.length; i++) {
          validateChildKeys(arguments[i], newElement.type);
        }
        validatePropTypes(newElement);
        return newElement;
      }
      {
        try {
          var frozenObject = Object.freeze({});
          var testMap = /* @__PURE__ */ new Map([[frozenObject, null]]);
          var testSet = /* @__PURE__ */ new Set([frozenObject]);
          testMap.set(0, 0);
          testSet.add(0);
        } catch (e) {
        }
      }
      var createElement$1 = createElementWithValidation;
      var cloneElement$1 = cloneElementWithValidation;
      var createFactory = createFactoryWithValidation;
      var Children = {
        map: mapChildren,
        forEach: forEachChildren,
        count: countChildren,
        toArray,
        only: onlyChild
      };
      react_development.Children = Children;
      react_development.Component = Component;
      react_development.Fragment = REACT_FRAGMENT_TYPE;
      react_development.Profiler = REACT_PROFILER_TYPE;
      react_development.PureComponent = PureComponent;
      react_development.StrictMode = REACT_STRICT_MODE_TYPE;
      react_development.Suspense = REACT_SUSPENSE_TYPE;
      react_development.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = ReactSharedInternals;
      react_development.cloneElement = cloneElement$1;
      react_development.createContext = createContext;
      react_development.createElement = createElement$1;
      react_development.createFactory = createFactory;
      react_development.createRef = createRef;
      react_development.forwardRef = forwardRef;
      react_development.isValidElement = isValidElement2;
      react_development.lazy = lazy;
      react_development.memo = memo;
      react_development.useCallback = useCallback;
      react_development.useContext = useContext;
      react_development.useDebugValue = useDebugValue;
      react_development.useEffect = useEffect;
      react_development.useImperativeHandle = useImperativeHandle;
      react_development.useLayoutEffect = useLayoutEffect;
      react_development.useMemo = useMemo;
      react_development.useReducer = useReducer;
      react_development.useRef = useRef;
      react_development.useState = useState;
      react_development.version = ReactVersion;
    })();
  }
  return react_development;
}
if (process.env.NODE_ENV === "production") {
  requireReact_production_min();
} else {
  requireReact_development();
}
const loader = new AssetLoader();
const builtinComponents = { Slot, Leaf, Page };
function createDocumentInstance(document2, context) {
  const instancesMap = /* @__PURE__ */ new Map();
  const vueInstanceMap = /* @__PURE__ */ new Map();
  const timestamp = ref(Date.now());
  const schema = computed(() => document2.export(IPublicEnumTransformStage.Render)), checkInstanceMounted = (instance) => instance._isMounted;
  const setHostInstance = (docId, nodeId, instances) => {
    const instanceRecords = instances ? instances.map((inst) => createComponentRecord(docId, nodeId, inst._uid)) : null;
    host.setInstance(docId, nodeId, instanceRecords);
  };
  const getComponentInstance = (id) => vueInstanceMap.get(id);
  const mountInstance = (id, instanceOrEl) => {
    const docId = document2.id;
    if (instanceOrEl == null) {
      let instances2 = instancesMap.get(id);
      if (instances2) {
        instances2 = instances2.filter(checkInstanceMounted);
        if (instances2.length > 0) {
          instancesMap.set(id, instances2);
          setHostInstance(docId, id, instances2);
        } else {
          instancesMap.delete(id);
          setHostInstance(docId, id, null);
        }
      }
      return;
    }
    const el = instanceOrEl.$el;
    const origId = getCompRootData(el).nodeId;
    if (origId && origId !== id) {
      unmountInstance(origId, instanceOrEl);
    }
    onUnmounted(() => unmountInstance(id, instanceOrEl), instanceOrEl);
    setCompRootData(el, {
      nodeId: id,
      docId,
      instance: instanceOrEl
    });
    let instances = instancesMap.get(id);
    if (instances) {
      const l = instances.length;
      instances = instances.filter(checkInstanceMounted);
      let updated = instances.length !== l;
      if (!instances.includes(instanceOrEl)) {
        instances.push(instanceOrEl);
        updated = true;
      }
      if (!updated)
        return;
    } else {
      instances = [instanceOrEl];
    }
    vueInstanceMap.set(instanceOrEl._uid, instanceOrEl);
    instancesMap.set(id, instances);
    setHostInstance(docId, id, instances);
  };
  const unmountInstance = (id, instance) => {
    const instances = instancesMap.get(id);
    if (instances) {
      const i = instances.indexOf(instance);
      if (i > -1) {
        const [instance2] = instances.splice(i, 1);
        vueInstanceMap.delete(instance2._uid);
        setHostInstance(document2.id, id, instances);
      }
    }
  };
  const getNode = (id) => id ? document2.getNode(id) : null;
  return reactive({
    id: computed(() => document2.id),
    path: computed(() => {
      const { fileName } = document2;
      return fileName.startsWith("/") ? fileName : `/${fileName}`;
    }),
    key: computed(() => `${document2.id}:${timestamp.value}`),
    scope: computed(() => {
      var _a;
      const _schema = schema.value;
      const { utils, constants, ...others } = context.appHelper;
      return {
        utils: {
          ...utils,
          ...buildUtils(host.libraryMap, (_a = Reflect.get(_schema, "utils")) != null ? _a : [])
        },
        constants: {
          ...constants,
          ...Reflect.get(_schema, "constants")
        },
        ...others
      };
    }),
    schema,
    document: computed(() => document2),
    messages: computed(() => deepMerge(context.i18n, Reflect.get(schema.value, "i18n"))),
    instancesMap: computed(() => instancesMap),
    getNode,
    mountInstance,
    unmountIntance: unmountInstance,
    getComponentInstance,
    rerender: () => void (timestamp.value = Date.now())
  });
}
function createSimulatorRenderer() {
  const layout = shallowRef({});
  const device = shallowRef("default");
  const locale = shallowRef();
  const autoRender = shallowRef(host.autoRender);
  const designMode = shallowRef("design");
  const libraryMap = shallowRef({});
  const components = shallowRef({});
  const componentsMap = shallowRef({});
  const requestHandlersMap = shallowRef({});
  const documentInstances = shallowRef([]);
  const context = reactive({
    i18n: {},
    appHelper: {
      utils: {},
      constants: {}
    }
  });
  const disposeFunctions = [];
  const documentInstanceMap = /* @__PURE__ */ new Map();
  function _buildComponents() {
    components.value = {
      ...builtinComponents,
      ...buildComponents(
        libraryMap.value,
        componentsMap.value,
        simulator2.createComponent
      )
    };
  }
  const simulator2 = reactive({
    config: markRaw(config),
    layout,
    device,
    designMode,
    libraryMap,
    components,
    autoRender,
    componentsMap,
    documentInstances,
    requestHandlersMap,
    isSimulatorRenderer: true
  });
  Vue.use(VueRouter);
  const router = new VueRouter();
  simulator2.router = router;
  simulator2.app = new Vue({
    router,
    render: (h2) => h2(SimulatorRendererView, {
      props: {
        simulator: simulator2
      }
    })
  });
  simulator2.getComponent = (componentName) => {
    const paths = componentName.split(".");
    const subs = [];
    for (; paths.length > 0; ) {
      const component = components.value[componentName];
      if (component)
        return getSubComponent(component, subs);
      const sub = paths.pop();
      if (!sub)
        break;
      subs.unshift(sub);
      componentName = paths.join(".");
    }
    return null;
  };
  simulator2.getClosestNodeInstance = (el, specId) => {
    var _a;
    if (isComponentRecord(el)) {
      const { cid, did } = el;
      const documentInstance = documentInstanceMap.get(did);
      const instance = (_a = documentInstance == null ? void 0 : documentInstance.getComponentInstance(cid)) != null ? _a : null;
      return instance && getClosestNodeInstanceByComponent(instance, specId);
    }
    return getClosestNodeInstance(el, specId);
  };
  simulator2.findDOMNodes = (instance) => {
    if (instance) {
      const { did, cid } = instance;
      const documentInstance = documentInstanceMap.get(did);
      const compInst = documentInstance == null ? void 0 : documentInstance.getComponentInstance(cid);
      return compInst ? findDOMNodes(compInst) : null;
    }
    return null;
  };
  simulator2.getClientRects = (element) => getClientRects(element);
  simulator2.setNativeSelection = (enable) => setNativeSelection(enable);
  simulator2.setDraggingState = (state) => cursor.setDragging(state);
  simulator2.setCopyState = (state) => cursor.setCopy(state);
  simulator2.clearState = () => cursor.release();
  simulator2.createComponent = () => null;
  simulator2.rerender = () => documentInstances.value.forEach((doc) => doc.rerender());
  simulator2.dispose = () => {
    simulator2.app.$destroy();
    disposeFunctions.forEach((fn) => fn());
  };
  simulator2.getCurrentDocument = () => {
    const crr = host.project.currentDocument;
    return documentInstances.value.find((doc) => doc.id === crr.id);
  };
  simulator2.load = (assets) => loader.load(assets);
  simulator2.loadAsyncLibrary = async (asyncLibraryMap) => {
    await loader.loadAsyncLibrary(asyncLibraryMap);
    _buildComponents();
  };
  let running = false;
  simulator2.run = () => {
    if (running)
      return;
    running = true;
    const containerId = "app";
    let container = document.getElementById(containerId);
    if (!container) {
      container = document.createElement("div");
      document.body.appendChild(container);
      container.id = containerId;
    }
    document.documentElement.classList.add("engine-page");
    document.body.classList.add("engine-document");
    simulator2.app.$mount(container);
    host.project.setRendererReady(simulator2);
  };
  disposeFunctions.push(
    //有 Renderer 进程连接进来，设置同步机制，自动运行第二个函数参数
    host.connect(simulator2, () => {
      var _a;
      const config22 = host.project.get("config") || {};
      layout.value = (_a = config22.layout) != null ? _a : {};
      if (libraryMap.value !== host.libraryMap || componentsMap.value !== host.designer.componentsMap) {
        libraryMap.value = host.libraryMap || {};
        componentsMap.value = host.designer.componentsMap;
        _buildComponents();
      }
      locale.value = host.locale;
      device.value = host.device;
      designMode.value = host.designMode;
      requestHandlersMap.value = host.requestHandlersMap;
    })
  );
  disposeFunctions.push(
    host.autorun(() => {
      const { router: router2 } = simulator2;
      const queryParams = parseParentQueryParams();
      documentInstances.value = host.project.documents.map((doc) => {
        let documentInstance = documentInstanceMap.get(doc.id);
        if (!documentInstance) {
          documentInstance = createDocumentInstance(doc, context);
          documentInstanceMap.set(doc.id, documentInstance);
          router2.addRoute({
            name: documentInstance.id,
            path: documentInstance.path,
            component: Renderer,
            props: () => ({
              key: documentInstance == null ? void 0 : documentInstance.key,
              documentInstance,
              simulator: simulator2
            })
          });
        }
        return documentInstance;
      });
      router2.beforeEach((to, from, next) => {
        var _a, _b;
        if (to.matched.length) {
          Object.assign(to.query, queryParams);
          next();
        } else {
          next(false);
          const tip = "编辑态不允许路由跳转";
          console.error(tip);
          if (window.vant) {
            window.vant.Toast(tip);
          } else if ((_a = window.ELEMENT) == null ? void 0 : _a.Message) {
            (_b = window.ELEMENT) == null ? void 0 : _b.Message.warning(tip);
          }
        }
      });
      router2.getRoutes().forEach((route) => {
        const id = route.name;
        documentInstances.value.some((doc) => doc.id === id) || documentInstanceMap.delete(id);
      });
      const inst = simulator2.getCurrentDocument();
      inst && inst.id !== router2.currentRoute.name && router2.replace({ name: inst.id });
    })
  );
  host.componentsConsumer.consume(async (componentsAsset) => {
    if (componentsAsset) {
      await loader.load(componentsAsset);
      _buildComponents();
    }
  });
  host.injectionConsumer.consume((data) => {
    var _a;
    if (data.appHelper) {
      const { utils, constants, ...other } = data.appHelper;
      Object.assign(context.appHelper, {
        utils: Array.isArray(utils) ? buildUtils(host.libraryMap, utils) : utils != null ? utils : {},
        constants: constants != null ? constants : {},
        ...other
      });
    }
    context.i18n = (_a = data.i18n) != null ? _a : {};
  });
  return simulator2;
}
function parseParentQueryParams() {
  const parentUrl = window.parent.location.href;
  const queryString = parentUrl.substring(parentUrl.lastIndexOf("?") + 1);
  const queryParams = {};
  const paramRegex = /([^?&=]+)=([^?&=]*)/g;
  queryString.replace(paramRegex, function(match, key, value) {
    const decodedKey = decodeURIComponent(key);
    let decodedValue = decodeURIComponent(value);
    decodedValue = String(decodedValue);
    queryParams[decodedKey] = decodedValue;
    return match;
  });
  return queryParams;
}
const simulator = createSimulatorRenderer();
const index = "";
const win = window;
if (typeof win !== "undefined") {
  win.SimulatorRenderer = simulator;
}
window.addEventListener("beforeunload", () => {
  win.LCSimulatorHost = null;
  win.SimulatorRenderer = null;
  simulator.dispose();
});
const name = "@mfejs/lowcode-vue-simulator-renderer2.7";
const version = "1.0.0";
console.log(
  `%c ${name} %c v`.concat(version, " "),
  "padding: 2px 1px; border-radius: 3px 0 0 3px; color: #fff; background: #5584ff; font-weight: bold;",
  "padding: 2px 1px; border-radius: 0 3px 3px 0; color: #fff; background: #42c02e; font-weight: bold;"
);
export {
  default2 as VueRenderer,
  simulator as default,
  config2 as vueRendererConfig
};
//# sourceMappingURL=vue-simulator-renderer.mjs.map
